#!/bin/sh
# Copyright (c) 2002-2011 by Rack-Soft Inc.
# Copyright (c) 2002-2011 by 4PSA (www.4psa.com)
# All rights reserved


# Modified: $DateTime: 2013/02/09 19:15:52 $
# Revision: $Revision: #33 $ with $Change: 96419 $

# DUMPS DNS Zones in the 4PSA DNS Manager 3 dump format
# Compatible with standard bind installations
# Compatible with DNS Manager 3.0.0 and above
# Description: for every zone listed in $named_conf, the script parse and dumps DNS records

# --------------------------------------------------------------------------------
#
# Please modify the default DNS records target (backup file)
# This will be remote update location in 4PSA DNS Manager
# You must include the full path!
# The program can also take a dump destination as the argument

# NOTE that if 1 argument is given dump_file will be the argument
dump_file="dump_full_recs.txt"
dump_temp_file="$dump_file-$$"

# Depending on zone configuration you need chroot or not
#chroot_dir="/var/named/named/run-root"
# p.a chroot_dir="/var/named/run-root" 
# Set it so that chroot_dir+file statement in named.conf == absolute file of zone file 
chroot_dir="/var/named/run-root/var"

# You can modify the following parameter to match your environment (use absolute path here)
named_conf="/etc/named.conf"

# Dump type can be <masters/slaves/both>
dump="both"

# Converts master zones to slaves in dump (for DNS Manager 3 acting as slave for a bind server)
# Works only when
# dump=masters/both
# dump_allow_transfers=yes/no
# dump_masters=yes/no
masters2slaves="no"

# Dump 'master' records from existing zones and includes them in the dump
# Works only when:
# dump=slaves/both
# masters2slaves=yes/no (Zones transformed from master2slave can not have a
# masters record because it does not exist in database!)
# dump_allow_transfers="yes"
dump_masters="yes"

# Dump 'allow-transfer' servers from existing zones and includes them in the dump
# Works only when:
# dump=master/both
# dump_masters=yes/no
# masters2slaves=yes/no
dump_allow_transfers="yes"

# Dump reverse zones
dump_reverse="yes"

# Dump soa records 
dump_soa="yes"

#exclude zone file (one zone per line) 
exclude_zone=""

# DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU KNOW WHAT YOU ARE DOING
# -------------------------------------------------------------------------

get_config()
{
if [ ! -f ${named_conf} ]; then
        echo "Config file does not exist, exiting."
        exit 1
fi
}

is_root()
{
        current_user="`id`"
        case ${current_user} in
                uid=0\(root\)*)
                                ;;
                            *)
                                echo "$0: You must have root privileges to run this script"
                                echo "Su as root and try again"
                                echo
                                exit 1
                                ;;
        esac
}

determine_mask()
# determine network mask for reverse zones
# $1 is reverse zone name
# returns result inglobal variable $mask
{
        local r_ip
        local ip

        r_ip="`echo "$1" | awk '{ pos=index(tolower($0), ".in-addr.arpa"); ip=substr($0, 1, pos - 1);  print ip}'` "
        if [ ! -z ${r_ip} ]; then
                mask="`echo ${r_ip} | awk 'BEGIN{FS="."}{print NF}'`"
                if [ ! -z ${mask} ]; then
                        return 0
                fi
        fi
return 1
}

get_masters(){
        if [ "$masters2slaves" = "yes" ];then
                local zonetype="slave"
        else
                local zonetype="master"
        fi
        if [ "$dump_reverse" = "yes" ];then
        # REVERSE ZONES
                cat ${named_conf}|sed 's/#.*$//g'|tr -s "\n \t" " "|grep -Eo "[ ]*zone.*};"|sed 's/\( zone[ ]*\)/\x0A\1/g;s/{/ {/g'|awk '{gsub("x0A","\n");print}'|awk '(tolower($2)~/^\".+\.in-addr\.arpa\"$/)&&($0~/type[ ]*master/)&&($0~/file[ ]*"[^"]*"/){print $0}' |while read LINE; do
                        ZNAME="`echo $LINE|awk '{gsub(/"/,"",$2);print $2}'|sed 's/ //g'|sed 's/\.$//'`"
                        rfile="$chroot_dir/$(echo $LINE|grep -Eo "[ ]+file[ ]+\"[^\"]*\""|awk '{buff=gsub(/"/,"",$2);print $buff}'|sed 's/ //g')"
        		if [ -f "${exclude_zone}" ];then
	                        if [ `grep -c ${ZNAME} ${exclude_zone}` -gt 0 ];then
					echo "==> Excluding $ZNAME"
        	                        continue
                	        fi
	                fi

                        if [ ! -f ${rfile} ]; then
                                echo "Error: could not determine path to file for zone $ZNAME."
			else
                                # filed is divided in 5 of 6 parts
                                determine_mask "$ZNAME"
                                if [ $? -eq 0 ]; then
                                        r_ip="`echo $ZNAME | awk '{if (tolower($1) ~ /^.+\.in-addr\.arpa$/){ pos=index(tolower($0), ".in-addr.arpa"); ip=substr($0, 1, pos - 1);  print ip}}'` "
                                        if [ ! -z ${r_ip} ]; then
                                                #difer=`expr 4 - $mask`
                                                difer=4
                                                ip="`echo ${r_ip} | awk 'BEGIN{FS=".";ORS=" "}{for(i=NF; i>1; i--) printf("%s.",$i); printf("%s",$1)}' mask=${difer}`"
                                                bit_mask="`expr ${mask} \* 8`"
                                                echo "$ZNAME.|$zonetype {" >> ${dump_temp_file}
                                                if [ "$dump_allow_transfers" = "yes" ];then
                                                        local ALLOWTRANSFERS="`echo $LINE|grep -Eo "allow-transfer[\t ]*{[^}]*}"|grep -Eo "([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[\t ]*;|[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+\/[0-9]+;)"|tr -s ';' ' '`"
                                                        local ALLOWTRANSFER=""
                                                        for ALLOWTRANSFER in $ALLOWTRANSFERS ; do
                                                                echo "|ALLOW_TRANSFER| |$ALLOWTRANSFER| || ||">>${dump_temp_file}
                                                        done
                                                fi
						if [ "$dump_soa" = "yes" ];then 
							soa=`awk '{gsub(/;.*$/,"");gsub(/[ \t]+/," ");printf("%s",$0) }' ${rfile}|awk '{soa=match($0,/\([ \t]*[0-9]+[ \t]+[0-9wWdDhHmM]+[ \t]+[0-9wWdDhHmM]+[ \t]+[0-9wWdDhHmM]+[ \t]+[0-9wWdDhHmM]+[ \t]*\)/);soa=substr($0,RSTART,RLENGTH);gsub(/([\(\)]+)/,"",soa);gsub(/^[ \t]+/,"",soa);gsub(/[ \t]+$/,"",soa);print soa}'`
							icount=1
							for i in $soa ; do
								value=$(echo ${i}|awk '{if ( $0 ~ /([wW]|[dD]|[hH]|[mM])/ ){gsub(/[wW]/," \\* 604800 + ");gsub(/[Dd]/," \\* 86400 + ");gsub(/[Hh]/," \\* 3600 + ");gsub(/[mM]/," \\* 60 + ");print $0"0"}else{print}}'|xargs expr 2>/dev/null)
								case $icount in
							                1)
							                        SERIAL="$value"
							                ;;
							                2)
							                        REFRESH="$value"
							                ;;
							                3)
							                        RETRY="$value"
							                ;;
							                4)
							                        EXPIRE="$value"
							                ;;
							                5)
							                        MINTTL="$value"
							                ;;
							                *)
							                        echo "Error"
						                ;;
							        esac
							        icount=$(($icount + 1))
							done
							TTL=$(echo "$(awk '{gsub(/;.*$/,"");print}' ${rfile}|awk '/[Tt][Tt][lL][ \t]*([0-9]|[wW]|[dD]|[hH]|[mM])+/{if ( $2 ~ /([wW]|[dD]|[hH]|[mM])/ ){gsub(/[wW]/," \\* 604800 + ",$2);gsub(/[Dd]/," \\* 86400 + ",$2);gsub(/[Hh]/," \\* 3600 + ",$2);gsub(/[mM]/," \\* 60 + ",$2);print $2"0"}else{print $2}}')"|xargs expr 2>/dev/null)
							[ -n "${SERIAL}" ] && echo "|SERIAL| |$SERIAL| || ||" >> ${dump_temp_file}
							[ -n "${TTL}" ] && echo "|DEFAULT_TTL| |$TTL| || ||"  >> ${dump_temp_file}
							[ -n "${REFRESH}" ] && echo "|REFRESH| |$REFRESH| || ||" >> ${dump_temp_file}
							[ -n "${RETRY}" ] && echo "|RETRY| |$RETRY| || ||"  >> ${dump_temp_file}
							[ -n "${EXPIRE}" ] && echo "|EXPIRE| |$EXPIRE| || ||"  >> ${dump_temp_file}
							[ -n "${MINTTL}" ] && echo "|MIN_TTL| |$MINTTL| || ||"  >> ${dump_temp_file}
	

						fi
						primary_ns=`awk 'BEGIN{IGNORECASE=1;}/soa/{
					        		if (toupper($1)=="SOA"){gsub(/ \t/,"",$2);print $2;next};
					                	if (toupper($2)=="SOA"){gsub(/ \t/,"",$3);print $3;next};
					                        if (toupper($3)=="SOA"){gsub(/ \t/,"",$4);print $4;next};
				        		     }' ${rfile}`
				        	[ -n "${primary_ns}" ]||primary_ns="this_is_just_a_default"

                                                #dump only PTR and NS
                                                if [ "$masters2slaves" = "no" ];then
                                                        buff="`grep -Ev "^[ \t]*;" ${rfile}|tr -s "\r" "\n"|sed 's/;.*//'|awk -v primary_ns="${primary_ns}" '
                                                        {
                                                        	# Handle TXT
                                                        	#name  ttl  class   rr     text
                                                        	#joe        IN      TXT    "text"
                                                        	
                                                                if (toupper($4)=="TXT"){txt=substr($0,index($0,$5));gsub(/"/,"",txt); printf ("|%s| |%s| |%s| ||\n",toupper($4),$1,txt);next};
                                                                if (toupper($3)=="TXT"){txt=substr($0,index($0,$4));gsub(/"/,"",txt); printf ("|%s| |%s| |%s| ||\n",toupper($3),$1,txt);next};
                                                                if (toupper($1)=="TXT"){txt=substr($0,index($0,$2));gsub(/"/,"",txt); printf ("|%s| || |%s| ||\n",toupper($1),txt);next};

								# Handle Origin 
                                                                if ((NF==2)&&(toupper($1) ~ /ORIGIN/)) { printf("|$ORIGIN| |%s| || ||\n", $2);next}
								
								# Handle NS
								#name  ttl  class   rr     canonical name 
								#example.com.        IN      NS  ns1.example.com. 

								#bla.com.      	     IN      NS  ns1.example.com.
								if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="NS")) {if (toupper($4)==toupper(primary_ns)){ printf("|%s| |%s| |%s| |1|\n", toupper($3),$1,$4);next}; printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next }
								#IN      NS  ns1.example.com.
								if ((NF==3)&&(toupper($1)=="IN")&&(toupper($2)=="NS")) {if (toupper($3)==toupper(primary_ns)){printf("|%s| || |%s| |1|\n", toupper($2),$3);next};printf("|%s| || |%s| ||\n", toupper($2),$3);next}
	   							#www	4324	IN      NS  ns1.example.com.
							   	if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="NS")) {if (toupper($5)==toupper(primary_ns)){printf("|%s| |%s| |%s| |1|\n", toupper($4),$1,$5);next};printf("|%s| |%s| |%s| ||\n", toupper($4),$1,$5);next}
	   	 						#NS  joe.example.com.
				   			        if ((NF==2)&&(toupper($1)!="IN")&&(toupper($1)=="NS")) {if (toupper($2)==toupper(primary_ns)){printf("|%s| || |%s| |1|\n", toupper($1),$2);next};printf("|%s| || |%s| ||\n", toupper($1),$2);next}
				   			        #www   		NS  ns1.example.com.
				   			        if ((NF==3)&&(toupper($1)!="IN")&&(toupper($2)=="NS")) {if (toupper($3)==toupper(primary_ns)){printf("|%s| |%s| |%s| |1|\n", toupper($2),$1,$3);next};printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
				   			        #www   43243		NS  ns1.example.com.
				   			        if ((NF==4)&&(toupper($2)!="IN")&&(toupper($3)=="NS")) {if (toupper($4)==toupper(primary_ns)){printf("|%s| |%s| |%s| |1|\n", toupper($3),$1,$4);next};printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}

								

								
								# Handle CNAME records
								#name  ttl  class   rr     canonical name 
								#www        IN      CNAME  joe.example.com. 
                                                                if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="CNAME")&&(toupper($4)!="TXT")) {printf("|%s| |%s| |%s| ||\n", toupper($4),$1,$5);next}
                                                                if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="CNAME")&&(toupper($3)!="TXT")) {printf("|%s| |%s| |%s| |%s|\n", toupper($3),$1,$4,"'$bit_mask'");next}
                                                                if ((NF==3)&&(toupper($2)=="CNAME")&&(toupper($1)!="IN"))  {printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
                                                                if ((NF==3)&&(toupper($2)=="CNAME")&&(toupper($1)=="IN"))  {printf("|%s| || |%s| ||\n", toupper($2),$3);next}

								# Handle PTR records
								#name ttl  class   rr     name
								#15         IN     PTR    www.example.com.
								
								# PTR  bla.com.
                                                                if ((NF==2)&&(toupper($1)=="PTR")) { printf("|%s| || |%s| ||\n", toupper($1),$2);next}              
                                                                # 15 PTR bla.com
                                                                if ((NF==3)&&(toupper($2)=="PTR")&&(toupper($1)!="IN"))  {printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
                                                                #15	54   IN  PTR bla.com
                                                                if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="PTR")) {printf("|%s| |%s| |%s| |%s|\n", toupper($4),$1,$5,"'$bit_mask'");next}
                                                                #15  IN PTR bla.com
                                                                if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="PTR")) {printf("|%s| |%s| |%s| |%s|\n", toupper($3),$1,$4,"'$bit_mask'");next}


                                                        }
                                                        '`"
                                                        echo "${buff}" >> ${dump_temp_file}
                                                fi
                                                echo "}" >> ${dump_temp_file}
                                        else
                                                echo "Error: unknown PTR record structure for reverse zone $ZNAME"
                                        fi
                                else
                                        echo "Error: invalid reverse zone name $ZNAME"
                                fi
                        fi
                done
        fi
        # FORWARD ZONES
        cat ${named_conf}|sed 's/#.*$//g'|tr -s "\n \t" " "|grep -Eo "[ ]*zone.*};"|sed 's/\( zone[ ]*\)/\x0A\1/g;s/{/ {/g'|awk '{gsub("x0A","\n");print}'|awk '(tolower($2)!~/^\".+\.in-addr\.arpa\"$/)&&($0~/type[ ]*master/)&&($0~/file[ ]*"[^"]*"/){print $0}' |while read LINE; do
                ZNAME="`echo $LINE|awk '{gsub(/"/," ",$2);print $2}'|sed 's/ //g'|sed 's/\.$//'`"
                file="$chroot_dir/$(echo $LINE|grep -Eo "[ ]+file[ ]+\"[^\"]*\""|awk '{gsub(/"/," ",$2);print $2}'|sed 's/ //g')"
        	if [ -f "${exclude_zone}" ];then
	        	if [ `grep -c ${ZNAME} ${exclude_zone}` -gt 0 ];then
				echo "==> Excluding $ZNAME"
        	                continue
                	fi
                fi
		
                if [ ! -f ${file} ]; then
                        echo "Error: could not determine path to file for zone $ZNAME."
                else
                        echo "$ZNAME.|$zonetype {" >> ${dump_temp_file}
                        if [ "$dump_allow_transfers" = "yes" ];then
                                local ALLOWTRANSFERS="`echo $LINE|grep -Eo "allow-transfer[\t ]*{[^}]*}"|grep -Eo "([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[\t ]*;|[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+\/[0-9]+;)"|tr -s ';' ' '`"
                                local ALLOWTRANSFER=""
                                for ALLOWTRANSFER in $ALLOWTRANSFERS ; do
                                        echo "|ALLOW_TRANSFER| |$ALLOWTRANSFER| || ||">>${dump_temp_file}
                                done
                        fi
                        if [ "$masters2slaves" = "no" ];then
				if [ "$dump_soa" = "yes" ];then 
					soa=`awk '{gsub(/;.*$/,"");gsub(/[ \t]+/," ");printf("%s",$0) }' ${file}|awk '{soa=match($0,/\([ \t]*[0-9]+[ \t]+[0-9wWdDhHmM]+[ \t]+[0-9wWdDhHmM]+[ \t]+[0-9wWdDhHmM]+[ \t]+[0-9wWdDhHmM]+[ \t]*\)/);soa=substr($0,RSTART,RLENGTH);gsub(/([\(\)]+)/,"",soa);gsub(/^[ \t]+/,"",soa);gsub(/[ \t]+$/,"",soa);print soa}'`
					icount=1
					for i in $soa ; do
					value=$(echo ${i}|awk '{if ( $0 ~ /([wW]|[dD]|[hH]|[mM])/ ){gsub(/[wW]/," \\* 604800 + ");gsub(/[Dd]/," \\* 86400 + ");gsub(/[Hh]/," \\* 3600 + ");gsub(/[mM]/," \\* 60 + ");print $0"0"}else{print}}'|xargs expr 2>/dev/null)
						case $icount in
					                1)
					                        SERIAL="$value"
					                ;;
					                2)
					                        REFRESH="$value"
					                ;;
					                3)
					                        RETRY="$value"
					                ;;
					                4)
					                        EXPIRE="$value"
					                ;;
					                5)
					                        MINTTL="$value"
					                ;;
					                *)
					                        echo "Error"
				                ;;
					        esac
					        icount=$(($icount + 1))
					done
					TTL=$(echo "$(awk '{gsub(/;.*$/,"");print}' ${file}|awk '/[Tt][Tt][lL][ \t]*([0-9]|[wW]|[dD]|[hH]|[mM])+/{if ( $2 ~ /([wW]|[dD]|[hH]|[mM])/ ){gsub(/[wW]/," \\* 604800 + ",$2);gsub(/[Dd]/," \\* 86400 + ",$2);gsub(/[Hh]/," \\* 3600 + ",$2);gsub(/[mM]/," \\* 60 + ",$2);print $2"0"}else{print $2}}')"|xargs expr 2>/dev/null)

					[ -n "${SERIAL}" ] && echo "|SERIAL| |$SERIAL| || ||" >> ${dump_temp_file}
					[ -n "${TTL}" ] && echo "|DEFAULT_TTL| |$TTL| || ||"  >> ${dump_temp_file}
					[ -n "${REFRESH}" ] && echo "|REFRESH| |$REFRESH| || ||" >> ${dump_temp_file}
					[ -n "${RETRY}" ] && echo "|RETRY| |$RETRY| || ||"  >> ${dump_temp_file}
					[ -n "${EXPIRE}" ] && echo "|EXPIRE| |$EXPIRE| || ||"  >> ${dump_temp_file}
					[ -n "${MINTTL}" ] && echo "|MIN_TTL| |$MINTTL| || ||"  >> ${dump_temp_file}
				fi
				primary_ns=`awk 'BEGIN{IGNORECASE=1;}/soa/{
				        		if (toupper($1)=="SOA"){gsub(/ \t/,"",$2);print $2;next};
				                	if (toupper($2)=="SOA"){gsub(/ \t/,"",$3);print $3;next};
				                        if (toupper($3)=="SOA"){gsub(/ \t/,"",$4);print $4;next};
				             }' ${file}`
		        	[ -n "${primary_ns}" ]||primary_ns="this_is_just_a_default"
				
                                buff="`grep -Ev "^[ \t]*;" ${file}|tr -s "\r" "\n"|sed 's/;.*//'|awk -v primary_ns="${primary_ns}" '
                                {
					# Handle TXT records
					#name  ttl  class   rr     text
					#joe        IN      TXT    "Located in a black hole"
					
	                                if (toupper($4)=="TXT"){txt=substr($0,index($0,$5));gsub(/"/,"",txt); printf ("|%s| |%s| |%s| ||\n",toupper($4),$1,txt);next};
                                        if (toupper($3)=="TXT"){txt=substr($0,index($0,$4));gsub(/"/,"",txt); printf ("|%s| |%s| |%s| ||\n",toupper($3),$1,txt);next};
                                        if (toupper($1)=="TXT"){txt=substr($0,index($0,$2));gsub(/"/,"",txt); printf ("|%s| || |%s| ||\n",toupper($1),txt);next};

	                                if (toupper($4)=="SPF"){txt=substr($0,index($0,$5));gsub(/"/,"",txt); printf ("|TXT| |%s| |%s| ||\n",$1,txt);next};
                                        if (toupper($3)=="SPF"){txt=substr($0,index($0,$4));gsub(/"/,"",txt); printf ("|TXT| |%s| |%s| ||\n",$1,txt);next};
                                        if (toupper($1)=="SPF"){txt=substr($0,index($0,$2));gsub(/"/,"",txt); printf ("|TXT| || |%s| ||\n",txt);next};
					
					# Handle SRV
					#srvce.prot.name  	ttl  class   rr  pri  weight port target
					#_http._tcp.example.com.	IN  SRV  0    5      80   www.example.com.
					
					#_http._tcp.example.com. 43243	 IN SRV  0    5      80   www.example.com.
		                        if ((NF==8)&&(toupper($4)=="SRV")&&(toupper($3)=="IN"))  {printf("|%s| |%s| |%s| |%s| |%s| |%s| |%s|\n", toupper($4),$1,$8,$5,$6,$7,$2);next}

					#_http._tcp.example.com.  IN SRV  0    5      80   www.example.com.
		                        if ((NF==7)&&(toupper($3)=="SRV")&&(toupper($2)=="IN"))  {printf("|%s| |%s| |%s| |%s| |%s| |%s| || \n", toupper($3),$1,$7,$4,$5,$6);next}

					#  IN SRV  0    5      80   www.example.com.
		                        if ((NF==6)&&(toupper($2)=="SRV")&&(toupper($1)=="IN"))  {printf("|%s| || |%s| |%s| |%s| |%s| ||\n", toupper($2),$6,$3,$4,$5);next}

					#_http._tcp.example.com. 43243	 SRV  0    5      80   www.example.com.
		                        if ((NF==7)&&(toupper($3)=="SRV")&&(toupper($2)!="IN"))  {printf("|%s| |%s| |%s| |%s| |%s| |%s| |%s| \n", toupper($3),$1,$7,$4,$5,$6,$2);next}

					#_http._tcp.example.com. 	 SRV  0    5      80   www.example.com.
                                        if ((NF==6)&&(toupper($2)=="SRV")&&(toupper($1)!="IN"))  {printf("|%s| |%s| |%s| |%s| |%s| |%s| ||\n", toupper($2),$1,$6,$3,$4,$5);next}
                                        #NAPTR
					#	IN NAPTR	order	preference	"flag"	"E2U+services"	"!regex!replacement!" .
					if ((NF==8)&&(toupper($1)=="IN")&&(toupper($2)=="NAPTR")){service=$6;gsub(/"/,"",service);flag=$5;gsub(/"/,"",flag);target=$7;gsub(/"/,"",target); printf("|%s| || |%s| |%s| |%s| || || |%s| |%s|\n",toupper($2),target,$3,$4,service,flag);next}
					#host.<domain>.	IN NAPTR	order	preference	"flag"	"E2U+services"	"!regex!replacement!" .
					if ((NF==9)&&(toupper($2)=="IN")&&(toupper($3)=="NAPTR")){service=$7;gsub(/"/,"",service);flag=$6;gsub(/"/,"",flag);target=$8;gsub(/"/,"",target); printf("|%s| |%s| |%s| |%s| |%s| || || |%s| |%s|\n",toupper($3),$1,target,$4,$5,service,flag);next}
					#host.<domain>.	123 IN NAPTR	order	preference	"flag"	"E2U+services"	"!regex!replacement!" .
					if ((NF==10)&&(toupper($3)=="IN")&&(toupper($4)=="NAPTR")){service=$8;gsub(/"/,"",service);flag=$7;gsub(/"/,"",flag);target=$9;gsub(/"/,"",target); printf("|%s| |%s| |%s| |%s| |%s| || || |%s| |%s|\n",toupper($4),$1,target,$5,$6,service,flag);next}

					# Handle Origin
                                        if ((NF==2)&&(toupper($1) ~ /ORIGIN/))  printf("|$ORIGIN| |%s| || ||\n", $2);
                                        
                                        # Handle MX records
                                        #name           ttl  class   rr  pref name
                                        #example.com.        IN      MX  10   mail.example.com.


					#example.com.        IN      MX  10   mail.example.com.
					if ((NF==5)&&(toupper($2)=="IN")&&(toupper($3)=="MX")&&(toupper($3)!="TXT")) {printf("|%s| |%s| |%s| |%s|\n", toupper($3),$1,$5,$4);next};
					#example.com.    432    IN      MX  10   mail.example.com.
					if ((NF==6)&&(toupper($3)=="IN")&&(toupper($4)=="MX")&&(toupper($4)!="TXT")) {printf("|%s| |%s| |%s| |%s|\n", toupper($4),$1,$6,$5);next}
					#        IN      MX  10   mail.example.com.
					if ((NF==4)&&(toupper($1)=="IN")&&(toupper($2)=="MX")) {printf("|%s| || |%s| |%s|\n", toupper($2),$4,$3);next}
					#        IN      MX    mail.example.com.
					if ((NF==3)&&(toupper($1)=="IN")&&(toupper($2)=="MX"))  {printf("|%s| || |%s| |%s|\n", toupper($2),$3,$4);next}
					#      MX     mail.example.com.
					if ((NF==2)&&(toupper($1)!="IN")&&(toupper($1)=="MX"))  {printf("|%s| || |%s| ||\n", toupper($1),$2);next}
					# example.com       MX  10   mail.example.com.
					if ((NF==3)&&(toupper($1)!="IN")&&(toupper($1)=="MX"))  {printf("|%s| || |%s| |%s|\n", toupper($1),$3,$2);next}
					# example.com     MX  10   mail.example.com.
					if ((NF==4)&&(toupper($2)!="IN")&&(toupper($2)=="MX"))  {printf("|%s| |%s| |%s| |%s|\n", toupper($2),$1,$4,$3);next}
                                        #example.com.       432     MX      10 mail.example.com.
                                        if ((NF==5)&&((toupper($2)!="IN")&&(toupper($3)=="MX"))) {printf("|%s| |%s| |%s| |%s|\n", toupper($3),$1,$5,$4);next}


					# Handle CNAME
					#name  ttl  class   rr     canonical name 
					#www        IN      CNAME  joe.example.com. 

					#www        	IN      CNAME  joe.example.com.
					if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="CNAME")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}
					#IN      CNAME  joe.example.com.
					if ((NF==3)&&(toupper($1)=="IN")&&(toupper($2)=="CNAME")) {printf("|%s| || |%s| ||\n", toupper($2),$3);next}
	   				#www	4324	IN      CNAME  joe.example.com.
				   	if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="CNAME")) {printf("|%s| |%s| |%s| ||\n", toupper($4),$1,$5);next}
	   	 			#CNAME  joe.example.com.
	   			        if ((NF==2)&&(toupper($1)!="IN")&&(toupper($1)=="CNAME")) {printf("|%s| || |%s| ||\n", toupper($1),$2);next}
	   			        #www  		CNAME  joe.example.com.
	   			        if ((NF==3)&&(toupper($1)!="IN")&&(toupper($2)=="CNAME")) {printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
	   			        #www   43243		CNAME  joe.example.com.
	   			        if ((NF==4)&&(toupper($2)!="IN")&&(toupper($3)=="CNAME")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}
	   			                                               

					# Handle NS
					#name  ttl  class   rr     canonical name 
					#example.com.        IN      NS  ns1.example.com. 

					#bla.com.      	     IN      NS  ns1.example.com.
					if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="NS")) {if (toupper($4)==toupper(primary_ns)){ printf("|%s| |%s| |%s| |1|\n", toupper($3),$1,$4);next}; printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next }
					#IN      NS  ns1.example.com.
					if ((NF==3)&&(toupper($1)=="IN")&&(toupper($2)=="NS")) {if (toupper($3)==toupper(primary_ns)){printf("|%s| || |%s| |1|\n", toupper($2),$3);next};printf("|%s| || |%s| ||\n", toupper($2),$3);next}
	   				#www	4324	IN      NS  ns1.example.com.
				   	if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="NS")) {if (toupper($5)==toupper(primary_ns)){printf("|%s| |%s| |%s| |1|\n", toupper($4),$1,$5);next};printf("|%s| |%s| |%s| ||\n", toupper($4),$1,$5);next}
	   	 			#NS  joe.example.com.
	   			        if ((NF==2)&&(toupper($1)!="IN")&&(toupper($1)=="NS")) {if (toupper($2)==toupper(primary_ns)){printf("|%s| || |%s| |1|\n", toupper($1),$2);next};printf("|%s| || |%s| ||\n", toupper($1),$2);next}
	   			        #www   		NS  ns1.example.com.
	   			        if ((NF==3)&&(toupper($1)!="IN")&&(toupper($2)=="NS")) {if (toupper($3)==toupper(primary_ns)){printf("|%s| |%s| |%s| |1|\n", toupper($2),$1,$3);next};printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
	   			        #www   43243		NS  ns1.example.com.
	   			        if ((NF==4)&&(toupper($2)!="IN")&&(toupper($3)=="NS")) {if (toupper($4)==toupper(primary_ns)){printf("|%s| |%s| |%s| |1|\n", toupper($3),$1,$4);next};printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}


					# Handle A 
					#name  ttl  class   rr     ip
					#joe        IN      A      192.168.254.3
			
					#bla.com.      	     IN      A  1.2.3.4
					if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="A")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}
					#IN      A  192.168.254.3
					if ((NF==3)&&(toupper($1)=="IN")&&(toupper($2)=="A")) {printf("|%s| || |%s| ||\n", toupper($2),$3);next}
	   				#www	4324	IN      A  192.168.254.3
				   	if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="A")) {printf("|%s| |%s| |%s| ||\n", toupper($4),$1,$5);next}
	   	 			#A  192.168.254.3.
	   			        if ((NF==2)&&(toupper($1)!="IN")&&(toupper($1)=="A")) {printf("|%s| || |%s| ||\n", toupper($1),$2);next}
	   			        #www   	A  192.168.254.3
	   			        if ((NF==3)&&(toupper($1)!="IN")&&(toupper($2)=="A")) {printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
	   			        #www   43243		A  192.168.254.3
	   			        if ((NF==4)&&(toupper($2)!="IN")&&(toupper($3)=="A")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}

					# Handle AAAA
					#name  ttl  class   rr     ip
					#joe        IN      AAAA      2001:db8::3
			
					#bla.com.      	     IN      AAAA  2001:db8::3
					if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="AAAA")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}
					#IN      AAAA  192.168.254.3
					if ((NF==3)&&(toupper($1)=="IN")&&(toupper($2)=="AAAA")) {printf("|%s| || |%s| ||\n", toupper($2),$3);next}
	   				#www	4324	IN      AAAA  2001:db8::3
				   	if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="AAAA")) {printf("|%s| |%s| |%s| ||\n", toupper($4),$1,$5);next}
	   	 			#AAAA  2001:db8::3
	   			        if ((NF==2)&&(toupper($1)!="IN")&&(toupper($1)=="AAAA")) {printf("|%s| || |%s| ||\n", toupper($1),$2);next}
	   			        #www   	AAAA  2001:db8::3
	   			        if ((NF==3)&&(toupper($1)!="IN")&&(toupper($2)=="AAAA")) {printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
	   			        #www   43243		AAAA  2001:db8::3
	   			        if ((NF==4)&&(toupper($2)!="IN")&&(toupper($3)=="AAAA")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}



                                        #2001:db8::3                 IN      PTR  bla.com
                                        if ((NF==4)&&(toupper($2)=="IN")&&(toupper($3)=="PTR")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}
                                        #IN      AAAA  192.168.254.3
                                        if ((NF==3)&&(toupper($1)=="IN")&&(toupper($2)=="PTR")) {printf("|%s| || |%s| ||\n", toupper($2),$3);next}
                                        #2001:db8::3    4324    IN      PTR  bla.com
                                        if ((NF==5)&&(toupper($3)=="IN")&&(toupper($4)=="PTR")) {printf("|%s| |%s| |%s| ||\n", toupper($4),$1,$5);next}
                                        #PTR  bla.com.
                                        if ((NF==2)&&(toupper($1)!="IN")&&(toupper($1)=="PTR")) {printf("|%s| || |%s| ||\n", toupper($1),$2);next}
                                        #www    AAAA  2001:db8::3
                                        if ((NF==3)&&(toupper($1)!="IN")&&(toupper($2)=="PTR")) {printf("|%s| |%s| |%s| ||\n", toupper($2),$1,$3);next}
                                        #2001:db8::3   43243            PTR  bla.com
                                        if ((NF==4)&&(toupper($2)!="IN")&&(toupper($3)=="PTR")) {printf("|%s| |%s| |%s| ||\n", toupper($3),$1,$4);next}

					                                                                                       

                                }
                                '`"
                                echo "${buff}" >> ${dump_temp_file}
                        fi
                        echo "}" >> ${dump_temp_file}
                fi
        done

}

get_slaves(){
        local LINE=""
        cat ${named_conf}|tr -s "\n \t" " "|grep -Eo "[ ]*zone.*};"|sed 's/\( zone[ ]*\)/\x0A\1/g;s/{/ {/g'|awk '{gsub("x0A","\n");print}'|awk  '($2!~/^\".+\.in-addr\.arpa\"$/)&&($0~/type[ ]*slave/){print $0}' |while read LINE; do
                local ZNAME="`echo $LINE|awk '{gsub(/"/," ",$2);print $2}'|sed 's/ //g'`"
               if [ -f "${exclude_zone}" ];then
                        if [ `grep -c ${ZNAME} ${exclude_zone}` -gt 0 ];then
                                echo "==> Excluding $ZNAME"
                                continue
                        fi
                fi
                echo "$ZNAME.|slave {" >>${dump_temp_file}
                if [ "$dump_allow_transfers" = "yes" ];then
                        local ALLOWTRANSFERS="`echo $LINE|grep -Eo "allow-transfer[\t ]*{[^}]*}"|grep -Eo "([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[\t ]*;|[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+\/[0-9]+;)"|tr -s ';' ' '`"
                        local ALLOWTRANSFER=""
                        for ALLOWTRANSFER in $ALLOWTRANSFERS ; do
                                echo "|ALLOW_TRANSFER| |$ALLOWTRANSFER| || ||">>${dump_temp_file}
                        done
                fi
                if [ "$dump_masters" = "yes" ];then
                        local MASTERS="`echo $LINE|grep -Eo "masters[\t ]*{[^}]*}"|grep -Eo "[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[\t ]*;"|tr -s ';' ' '`"
                        local MASTER=""
                        for MASTER in $MASTERS ; do
                                echo "|MASTER| |$MASTER| || ||">>${dump_temp_file}
                        done
                fi
                echo "}" >>${dump_temp_file}
        done
        if [ "$dump_reverse" = "yes" ];then
        # REVERSE ZONES
                cat ${named_conf}|tr -s "\n \t" " "|grep -Eo "[ ]*zone.*};"|sed 's/\( zone[ ]*\)/\x0A\1/g;s/{/ {/g'|awk '{gsub("x0A","\n");print}'|awk  '($2~/^\".+\.in-addr\.arpa\"$/)&&($0~/type[ ]*slave/){print $0}' |while read LINE; do
                       local ZNAME="`echo $LINE|awk '{gsub(/"/," ",$2);print $2}'|sed 's/ //g'`"
        	       if [ -f "${exclude_zone}" ];then
	                        if [ `grep -c ${ZNAME} ${exclude_zone}` -gt 0 ];then
                        	        echo "==> Excluding $ZNAME"
                	                continue
        	                fi
	                fi

                        echo "$ZNAME.|slave {" >>${dump_temp_file}
                        if [ "$dump_allow_transfers" = "yes" ];then
                                local ALLOWTRANSFERS="`echo $LINE|grep -Eo "allow-transfer[\t ]*{[^}]*}"|grep -Eo "([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[\t ]*;|[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+\/[0-9]+;)"|tr -s ';' ' '`"
                                local ALLOWTRANSFER=""
                                for ALLOWTRANSFER in $ALLOWTRANSFERS ; do
                                        echo "|ALLOW_TRANSFER| |$ALLOWTRANSFER| || ||">>${dump_temp_file}
                                done
                        fi
                        if [ "$dump_masters" = "yes" ];then
                                local MASTERS="`echo $LINE|grep -Eo "masters[\t ]*{[^}]*}"|grep -Eo "[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[\t ]*;"|tr -s ';' ' '`"
                                local MASTER=""
                                for MASTER in $MASTERS ; do
                                        echo "|MASTER| |$MASTER| || ||">>${dump_temp_file}
                                done
                        fi
                        echo "}" >>${dump_temp_file}
                done
        fi

}

# main
is_root
get_config

# get parameters
if [ $# -eq 1 ]; then
        echo "Using $1 as a dump file\n"
        dump_file="$1"
        dump_temp_file="$dump_file-$$"

fi

if [ $# -gt 1 ]; then
        exit 1
fi

case "$dump" in
        masters)
                get_masters
        ;;
        slaves)
                get_slaves
        ;;
        both)
                get_masters
                get_slaves
        ;;
esac

mv $dump_temp_file $dump_file >>/dev/null 2>&1
echo "`grep -c "{" $dump_file` zones processed sucesfully"

exit 0
